#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include "xdio.h"
#include "util.h"
#include "drive.h"


#define IOCTL_STORAGE_QUERY_PROPERTY CTL_CODE(IOCTL_STORAGE_BASE, 0x0500, METHOD_BUFFERED, FILE_ANY_ACCESS)


typedef enum _STORAGE_QUERY_TYPE {
    PropertyStandardQuery = 0,
    PropertyExistsQuery,
    PropertyMaskQuery,
    PropertyQueryMaxDefined
} STORAGE_QUERY_TYPE, *PSTORAGE_QUERY_TYPE;

typedef enum _STORAGE_PROPERTY_ID {
    StorageDeviceProperty = 0,
    StorageAdapterProperty
} STORAGE_PROPERTY_ID, *PSTORAGE_PROPERTY_ID;

typedef enum _STORAGE_BUS_TYPE {
    BusTypeUnknown = 0x00,
    BusTypeScsi,
    BusTypeAtapi,
    BusTypeAta,
    BusType1394,
    BusTypeSsa,
    BusTypeFibre,
    BusTypeUsb,
    BusTypeRAID,
    BusTypeiScsi,
    BusTypeSas,
    BusTypeSata,
    BusTypeSd,
    BusTypeMmc,
    BusTypeMax,
    BusTypeMaxReserved = 0x7F
} STORAGE_BUS_TYPE, *PSTORAGE_BUS_TYPE;

typedef struct _STORAGE_PROPERTY_QUERY {
    STORAGE_PROPERTY_ID PropertyId;
    STORAGE_QUERY_TYPE QueryType;
    UCHAR AdditionalParameters[1];
} STORAGE_PROPERTY_QUERY, *PSTORAGE_PROPERTY_QUERY;

typedef struct _STORAGE_DEVICE_NUMBER {
    DEVICE_TYPE DeviceType;
    ULONG DeviceNumber;
    ULONG PartitionNumber;
} STORAGE_DEVICE_NUMBER, *PSTORAGE_DEVICE_NUMBER;

typedef struct _STORAGE_DEVICE_DESCRIPTOR {
    ULONG Version;
    ULONG Size;
    UCHAR DeviceType;
    UCHAR DeviceTypeModifier;
    BOOLEAN RemovableMedia;
    BOOLEAN CommandQueueing;
    ULONG VendorIdOffset;
    ULONG ProductIdOffset;
    ULONG ProductRevisionOffset;
    ULONG SerialNumberOffset;
    STORAGE_BUS_TYPE BusType;
    ULONG RawPropertiesLength;
    UCHAR RawDeviceProperties[1];
} STORAGE_DEVICE_DESCRIPTOR, *PSTORAGE_DEVICE_DESCRIPTOR;


UINT DriveGetType(char drv)
{
	int							ret;
	char						buf[BUFSIZE];
	STORAGE_DEVICE_DESCRIPTOR	sdd;
	STORAGE_PROPERTY_QUERY		query;
	DWORD						len;
	HANDLE						dev;

    sprintf(buf, "\\\\.\\%c:", drv);

	ret = GetDriveType(buf + 4);
	if (ret == DRIVE_REMOVABLE) {
		return DRIVE_REMOVABLE;
	}
	if (ret != DRIVE_FIXED) {
		return 0;
	}

	dev = CreateFile(
			buf,
			0,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_READONLY,
			NULL);
	if (dev == INVALID_HANDLE_VALUE) {
//		ErrorHandler("CreateFile");
		return 0;
	}

	query.PropertyId = StorageDeviceProperty;
	query.QueryType = PropertyStandardQuery;

	ret = DeviceIoControl(
			dev,
			IOCTL_STORAGE_QUERY_PROPERTY,
			&query,
			sizeof(query),
			&sdd,
			sizeof(sdd),
			&len,
			NULL);
	CloseHandle(dev);
	if (!ret) {
//		ErrorHandler("DeviceIoControl");
		return 0;
	}

	return (sdd.BusType == BusTypeUsb) ? DRIVE_REMOVABLE : DRIVE_FIXED;
}


char *DriveGrubStr(char drv) {
	static char				buf[BUFSIZE];
	HANDLE					dev;
	STORAGE_DEVICE_NUMBER	sdn;
	PARTITION_INFORMATION	pi;
	int						ret;
	int						len;
	xd_t					*xd;
	xde_t					xe;

	sprintf(buf, "\\\\.\\%c:", drv);

	dev = CreateFile(
			buf,
			GENERIC_READ,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			0,
			NULL);
	if (dev == INVALID_HANDLE_VALUE) {
		ErrorHandler("CreateFile");
		return NULL;
	}

	ret = DeviceIoControl(
			dev,
			IOCTL_STORAGE_GET_DEVICE_NUMBER,
			NULL,
			0,
			&sdn,
			sizeof(sdn),
			&len,
			NULL);
	if (!ret) {
//		ErrorHandler("DeviceIoControl");
		CloseHandle(dev);
		return NULL;
	}

	ret = DeviceIoControl(
			dev,
			IOCTL_DISK_GET_PARTITION_INFO,
			NULL,
			0,
			&buf,
			sizeof(buf),
			&len,
			NULL);
	if (!ret) {
//		ErrorHandler("DeviceIoControl");
		CloseHandle(dev);
		return NULL;
	}
	CloseHandle(dev);
	memcpy(&pi, buf, sizeof(pi));

	sprintf(buf, "(hd%d)", sdn.DeviceNumber);

	xd = xd_open(buf, 0);
	if (xd == NULL) {
		return NULL;
	}

	xe.cur = xe.nxt = 0xFF;
	while (!xd_enum(xd, &xe)) {
		if (xe.bse == (pi.StartingOffset.QuadPart >> 9)
				&& xe.len == (pi.PartitionLength.QuadPart >> 9)) {
			sprintf(buf, "(hd%u,%u)", sdn.DeviceNumber, xe.cur);
			xd_close(xd);
			return buf;
		}
	}
	xd_close(xd);

	return NULL;
}


void DriveDebug(void) {
	char					buf[BUFSIZE];
	HANDLE					dev;
	STORAGE_DEVICE_NUMBER	sdn;
	PARTITION_INFORMATION	pi;
	int						ret;
	int						len;
	xd_t					*xd;
	xde_t					xe;
	FILE					*f;
	int						i;
	char					drv;

	f = fopen("c:\\pup2usb_drive.log", "w");

	for (i = GetLogicalDrives() >> 2, drv = 'C'; i != 0; i >>= 1, drv++) {
		if (!DriveGetType(drv)) {
			continue;
		}

		sprintf(buf, "\\\\.\\%c:", drv);

		dev = CreateFile(
				buf,
				GENERIC_READ,
				FILE_SHARE_READ | FILE_SHARE_WRITE,
				NULL,
				OPEN_EXISTING,
				0,
				NULL);
		if (dev == INVALID_HANDLE_VALUE) {
            fprintf(f, "CreateFile(%s): %s", buf, GetLastErrorStr());
            continue;
		}

		ret = DeviceIoControl(
				dev,
				IOCTL_STORAGE_GET_DEVICE_NUMBER,
				NULL,
				0,
				&sdn,
				sizeof(sdn),
				&len,
				NULL);
		if (!ret) {
            fprintf(f, "DeviceIoControl(%s,IOCTL_STORAGE_GET_DEVICE_NUMBER): %s",
                buf, GetLastErrorStr());
			CloseHandle(dev);
            continue;
		}

		ret = DeviceIoControl(
				dev,
				IOCTL_DISK_GET_PARTITION_INFO,
				NULL,
				0,
				&buf,
				sizeof(buf),
				&len,
				NULL);
		if (!ret) {
            fprintf(f, "DeviceIoControl(%s,IOCTL_DISK_GET_PARTITION_INFO): %s",
                buf, GetLastErrorStr());
			CloseHandle(dev);
            continue;
		}
		CloseHandle(dev);

		memcpy(&pi, buf, sizeof(pi));

		fprintf(f, "[winapi] dev=%c:(d%up%u) offset=%u size=%u\n",
			drv,
			(UINT)sdn.DeviceNumber,
			(UINT)sdn.PartitionNumber,
			(UINT)(pi.StartingOffset.QuadPart >> 9),
			(UINT)(pi.PartitionLength.QuadPart >> 9));
	}

	for (i = 0; i < 10; i++) {
		sprintf(buf, "(hd%d)", i);
		xd = xd_open(buf, 0);
		if (xd == NULL) {
			continue;
		}

		xe.cur = xe.nxt = 0xFF;
		while (!xd_enum(xd, &xe)) {
			fprintf(f, "[xdio] dev=hd%c%u offset=%u size=%u\n",
				'a' + i,
				xe.cur + 1,
				(UINT)xe.bse,
				(UINT)xe.len);
		}
		xd_close(xd);
	}

	fclose(f);

	return;
}
