#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <assert.h>
#include <stdio.h>
#include <commdlg.h>
#include <commctrl.h>
#include <shlobj.h>
#include "regex.h"
#include "xdio.h"
#include "drive.h"
#include "util.h"
#include "resource.h"


HINSTANCE hInst;

HWND hwndMain;

HWND hwndInstall;

BOOL isInstalling;


static void DriveCboRefresh()
{
	DWORD	drives;
	HWND	hwndCBO;
	char	buf[BUFSIZE];

	hwndCBO = GetDlgItem(hwndMain, IDC_CBO_DRIVE);
	SendMessage(hwndCBO, CB_RESETCONTENT, 0, 0);

	strcpy(buf, "C:\\");
	for (drives = GetLogicalDrives() >> 2; drives != 0; drives >>= 1) {
		if ((drives & 1) && DriveGetType(buf[0])) {
			if (GetVolumeInformation(buf, buf + 4, 128 - 4, NULL, NULL, NULL, NULL, 0)) {
				buf[3] = '[';
				strcat(buf + 4, "]");
			}
			buf[2] = ' ';
			SendMessage(hwndCBO, CB_ADDSTRING, 0, (LPARAM)buf);
			buf[2] = '\\';
			buf[3] = 0;
		}
		buf[0]++;
	}
}


static char DriveCboGet()
{
    char buf[BUFSIZE];

	GetDlgItemText(hwndMain, IDC_CBO_DRIVE, buf, sizeof(buf));
    return buf[0];
}


static void DriveCboProc(int param)
{
	switch (param) {
		case CBN_DROPDOWN:
			DriveCboRefresh(hwndMain);
			break;
		case CBN_SELCHANGE:
			if (DriveGetType(DriveCboGet(hwndMain)) == DRIVE_FIXED) {
				SendDlgItemMessage(hwndMain, IDC_CHK_FORMAT, BM_SETCHECK, BST_UNCHECKED, 0);
				EnableWindow(GetDlgItem(hwndMain, IDC_CHK_FORMAT), FALSE);
			} else {
				EnableWindow(GetDlgItem(hwndMain, IDC_CHK_FORMAT), TRUE);
			}
			break;
	}
}


static char *IsoEdtGet()
{
    static char buf[BUFSIZE];

	GetDlgItemText(hwndMain, IDC_EDT_ISO, buf, sizeof(buf));
    return buf;
}


static BOOL FormatChkGet()
{
    return SendDlgItemMessage(hwndMain, IDC_CHK_FORMAT, BM_GETCHECK, 0, 0) == BST_CHECKED;
}


static void BrowseBtnClick()
{
	OPENFILENAME	ofn;
	char			buf[BUFSIZE];

	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize		= sizeof(ofn);
	ofn.hwndOwner		= hwndMain;
	ofn.lpstrFile		= buf;
	ofn.lpstrFile[0]	= '\0';
	ofn.nMaxFile		= sizeof(buf);
	ofn.lpstrFilter		= "ISO Images\0*.iso\0All\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFileTitle	= NULL;
	ofn.nMaxFileTitle	= 0;
	ofn.lpstrInitialDir	= NULL;
	ofn.Flags			= OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

	if (GetOpenFileName(&ofn)==TRUE) {
		SetDlgItemText(hwndMain, IDC_EDT_ISO, buf);
	}
}


static BOOL MainDlgCheck() {
	char	buf[BUFSIZE];
	char	*iso;
	int		ret;

    iso = IsoEdtGet();
	if (iso[0] == 0) {
		MessageBox(
			hwndMain,
			GetResStr(IDS_SELECTISO),
			GetResStr(IDS_ERROR),
			MB_ICONERROR | MB_OK);
		SetFocus(GetDlgItem(hwndMain, IDC_EDT_ISO));
		return FALSE;
	}

	ret = GetFileAttributes(iso);
	if (ret & (FILE_ATTRIBUTE_DIRECTORY	| FILE_ATTRIBUTE_OFFLINE
				| FILE_ATTRIBUTE_SYSTEM )) {
		ret = snprintf(buf, sizeof(buf),
                GetResStr(IDS_INVALIDISO),
				iso);
		assert(ret < sizeof(buf));

		MessageBox(
			hwndMain,
			buf,
			GetResStr(IDS_ERROR),
			MB_ICONERROR | MB_OK);
		SetFocus(GetDlgItem(hwndMain, IDC_EDT_ISO));
		return FALSE;
	}

	if (DriveCboGet(hwndMain) == 0) {
		MessageBox(
			hwndMain,
			GetResStr(IDS_SELECTDRIVE),
			GetResStr(IDS_ERROR),
			MB_ICONERROR | MB_OK);
		SetFocus(GetDlgItem(hwndMain, IDC_CBO_DRIVE));
		return FALSE;
	}

	return TRUE;
}


static void LogEdtAppend(const char *buf) {
	int ndx;
	int hwnd;

    hwnd = GetDlgItem(hwndInstall, IDC_EDT_LOG);
	ndx = GetWindowTextLength(hwnd);
	SendMessage(hwnd, EM_SETSEL, (WPARAM)ndx, (LPARAM)ndx);
	SendMessage(hwnd, EM_REPLACESEL, 0, (LPARAM)buf);
}


static void InstallPgbSet(int pos) {
	SendDlgItemMessage(hwndInstall, IDC_PGB_INSTALL, PBM_SETPOS, (WPARAM)pos, 0);
}


static void StopBtnClick(void) {
	isInstalling = FALSE;
	SetDlgItemText(hwndInstall, IDC_BTN_STOP, GetResStr(IDS_BTN_STOP2));
}


static BOOL InstallDlgLayout(int nWidth, int nHeight)
{
	int		border = 6;
	RECT	rc;
	HWND	hwnd;

	hwnd = GetDlgItem(hwndInstall, IDC_BTN_STOP);
	GetWindowRect(hwnd, &rc);
	rc.right	-= rc.left;
	rc.bottom	-= rc.top;

	MoveWindow(hwnd,
		nWidth - rc.right - border,
		nHeight - rc.bottom - border,
		rc.right,
		rc.bottom,
		TRUE);

	hwnd = GetDlgItem(hwndInstall, IDC_EDT_LOG);
	MoveWindow(hwnd,
		6,
		6,
		nWidth - border - border,
		nHeight - border - border - rc.bottom - border,
		TRUE);

	hwnd = GetDlgItem(hwndInstall, IDC_PGB_INSTALL);
	MoveWindow(hwnd,
		6,
		nHeight - rc.bottom - border + 1,
		nWidth - border - border - border - rc.right,
		rc.bottom - 2,
		TRUE);

	return TRUE;
}


static void FormatDrive(const char drv) {
	char	buf[BUFSIZE];
	int		ret;

	sprintf(buf, GetResStr(IDS_FORMAT), drv);

	LogEdtAppend(buf);

	ret = SHFormatDrive(hwndInstall, drv - 'A', SHFMT_ID_DEFAULT, SHFMT_OPT_FULL);
	switch (ret) {
		case SHFMT_NOFORMAT:
			LogEdtAppend(GetResStr(IDS_FAIL));
			StopBtnClick();
			break;
		case SHFMT_ERROR:
		case SHFMT_CANCEL:
			LogEdtAppend(GetResStr(IDS_CANCEL));
			StopBtnClick();
			break;
		default:
			LogEdtAppend(GetResStr(IDS_DONE));
			break;
	}

	return;
}


static void ExtractISO(const char *iso, const char drv) {
	HANDLE				hOut;
	char				buf[BUFSIZE];
	char				*list;
	char				*p;
	char				*err;
	int					size;
	int                 len;
	int					total;
	int					ret;
	regex_t				preg;
	regmatch_t          pmatch[10];
	PROCESS_INFORMATION	pi;
	ULARGE_INTEGER		space;

	LogEdtAppend(GetResStr(IDS_CHECKISO));

	ret = snprintf(buf, sizeof(buf),
			"7z l \"%s\" *.sfs vmlinuz initrd.gz",
			iso);
	assert(ret < sizeof(buf));

	list = ReadPipe(POpen(buf, &pi, FALSE));
	if (!GetExitCodeProcess(pi.hProcess, &ret)) {
		ErrorHandler("GetExitCodeProcess");
		ExitProcess(3);
	}
	CloseHandle(pi.hThread);
	CloseHandle(pi.hProcess);

	if (ret != 0) {
		LogEdtAppend(GetResStr(IDS_FAIL));
		LogEdtAppend(list);
		StopBtnClick();
		return;
	}

	ret = regcomp(&preg,
            " \\+\\([0-9]\\+\\) \\+[0-9]\\+ \\+[0-9]\\+ files, [0-9]\\+ folders",
            REG_ICASE);
    assert(ret == 0);

    ret = regexec(&preg, list, sizeof(pmatch) / sizeof(regmatch_t), pmatch, 0);
	if (ret == REG_NOMATCH) {
		LogEdtAppend(GetResStr(IDS_FAILREAD));
		LogEdtAppend(list);
		StopBtnClick();
		return;
	}

	list[pmatch[1].rm_eo]   = 0;
	total		        	= strtol(list + pmatch[1].rm_so, NULL, 10);
	size                    = 0;

	LogEdtAppend(GetResStr(IDS_DONE));

	if (!isInstalling) {
		return;
	}

	LogEdtAppend(GetResStr(IDS_CHECKFS));

	sprintf(buf, "chkdsk %c:", drv);
    ret = System(buf);
	if (ret == 0) {
		LogEdtAppend(GetResStr(IDS_DONE));
	} else {
		if (ret == 2) {
			LogEdtAppend(GetResStr(IDS_FAILSLIGHT));
		} else if (ret == 3) {
			LogEdtAppend(GetResStr(IDS_FAIL));
		} else {
			LogEdtAppend(GetResStr(IDS_FAILUNKNOWN));
		}
		sprintf(buf, GetResStr(IDS_CHKDSK), drv);
		LogEdtAppend(buf);
		if (ret == 3) {
			StopBtnClick();
			return;
		}
	}

	if (!isInstalling) {
		return;
	}

	LogEdtAppend(GetResStr(IDS_CHECKCAPACITY));

	sprintf(buf, "%c:\\", drv);
	if (!GetDiskFreeSpaceEx(buf, &space, NULL, NULL)) {
		LogEdtAppend(GetResStr(IDS_FAIL));
		StopBtnClick();
		return;
	}
	if (space.QuadPart < total + 1>>20) {
		sprintf(buf, GetResStr(IDS_FAILCAPACITY), total >> 20);
		LogEdtAppend(buf);
		StopBtnClick();
		return;
	}
	LogEdtAppend(GetResStr(IDS_DONE));

	if (!isInstalling) {
		return;
	}

    ret = regcomp(&preg,
            ":[0-9]\\+ \\+\\([0-9]\\+\\) \\+[0-9]\\+ \\+\\([^\r]\\+\\)\r",
            0);
	assert(ret == 0);

	p = list;
	while ((regexec(&preg, p, sizeof(pmatch) / sizeof(regmatch_t), pmatch, 0)
            != REG_NOMATCH) && isInstalling) {
        p[pmatch[1].rm_eo]  = 0;
        p[pmatch[2].rm_eo]  = 0;

		ret = snprintf(buf, sizeof(buf),
				GetResStr(IDS_COPY),
				drv, "puppy\\", p + pmatch[2].rm_so);
		assert(ret < sizeof(buf));
		LogEdtAppend(buf);

		ret = snprintf(buf, sizeof(buf),
				"7z e -y -o%c:\\puppy\\ \"%s\" \"%s\"",
				drv, iso, p + pmatch[2].rm_so);
		assert(ret < sizeof(buf));
		hOut = POpen(buf, &pi, FALSE);

		ret = snprintf(buf, sizeof(buf),
				"%c:\\puppy\\%s",
				drv, p + pmatch[2].rm_so);
		assert(ret < sizeof(buf));

		len = strtol(p + pmatch[1].rm_so, NULL, 10);
        p   += pmatch[0].rm_eo + 1;

		while (TRUE) {
			Sleep(250);

			ret = FileSize(buf);
			InstallPgbSet(90LL * (size + ret) / total);

			if (!isInstalling) {
				TerminateProcess(pi.hProcess, 1);
				CloseHandle(pi.hThread);
				CloseHandle(pi.hProcess);
				LogEdtAppend(GetResStr(IDS_CANCEL));
				break;
			}

			if (!GetExitCodeProcess(pi.hProcess, &ret)) {
				ErrorHandler("GetExitCodeProcess");
				ExitProcess(3);
			}
			if (ret != STILL_ACTIVE) {
				if (ret != 0) {
					LogEdtAppend(GetResStr(IDS_FAIL));
					err = ReadPipe(hOut);
					LogEdtAppend(err);
					HeapFree(GetProcessHeap(), 0, err);
				}

				if (len != FileSize(buf)) {
					LogEdtAppend(GetResStr(IDS_FAILFILE));
					StopBtnClick();
					return;
				}
				size += len;

				LogEdtAppend(GetResStr(IDS_DONE));

				CloseHandle(pi.hThread);
				CloseHandle(pi.hProcess);

				break;
			}
		}
	}

	HeapFree(GetProcessHeap(), 0, list);

	return;
}


static void MakeBootSysLinux(const char drv) {
	char				buf[BUFSIZE];
	char				*err;
	int					ret;
	PROCESS_INFORMATION	pi;

	GetModuleFileName(NULL, buf, sizeof(buf));
	err		= strrchr(buf, '\\');
	*err	= 0;
	if (!SetCurrentDirectory(buf)) {
		ErrorHandler("SetCurrentDirectory");
		ExitProcess(3);
	}

	ret = snprintf(buf, sizeof(buf),
			GetResStr(IDS_SYSLINUXCFG),
			drv);
	assert(ret < sizeof(buf));
	LogEdtAppend(buf);

	ret = snprintf(buf, sizeof(buf),
			"%c:\\syslinux.cfg",
			drv);
	assert(ret < sizeof(buf));

	if (!CopyFile("syslinux.cfg", buf, FALSE)) {
	    LogEdtAppend(GetResStr(IDS_FAIL));
		LogEdtAppend(GetLastErrorStr());
		LogEdtAppend("\r\n");
		StopBtnClick();
        return;
	}
	LogEdtAppend(GetResStr(IDS_DONE));

	InstallPgbSet(95);

	if (!isInstalling) {
		return;
	}

	LogEdtAppend(GetResStr(IDS_SYSLINUX));

	ret = snprintf(buf, sizeof(buf),
			"syslinux -f %c:",
			drv);
	assert(ret < sizeof(buf));

	err = ReadPipe(POpen(buf, &pi, TRUE));
	if (!GetExitCodeProcess(pi.hProcess, &ret)) {
		ErrorHandler("GetExitCodeProcess");
		ExitProcess(3);
	}
	CloseHandle(pi.hThread);
	CloseHandle(pi.hProcess);

	if (ret != 0) {
		LogEdtAppend(GetResStr(IDS_FAIL));
		LogEdtAppend(err);
		StopBtnClick();
		HeapFree(GetProcessHeap(), 0, err);

		return;
	}

	LogEdtAppend(GetResStr(IDS_DONE));

	HeapFree(GetProcessHeap(), 0, err);
	InstallPgbSet(100);

	return;
}


static void MakeBootGrub4Dos(char drv) {
	char				buf[BUFSIZE];
	char				*err;
	FILE				*f;
    long                pos;

	GetModuleFileName(NULL, buf, sizeof(buf));
	err		= strrchr(buf, '\\');
	*err	= 0;
	if (!SetCurrentDirectory(buf)) {
		ErrorHandler("SetCurrentDirectory");
		ExitProcess(3);
	}

	LogEdtAppend(GetResStr(IDS_GRUBSTR));
	if (!DriveGrubStr(drv)) {
		LogEdtAppend(GetResStr(IDS_FAIL));
		StopBtnClick();
		return;
	}
	LogEdtAppend(GetResStr(IDS_DONE));

    sprintf(buf, GetResStr(IDS_COPY),
        'C', "", "GRLDR");
	LogEdtAppend(buf);

	SetFileAttributes("C:\\GRLDR", FILE_ATTRIBUTE_NORMAL);
	if (!CopyFile("grldr", "C:\\GRLDR", FALSE)) {
	    LogEdtAppend(GetResStr(IDS_FAIL));
		LogEdtAppend(GetLastErrorStr());
		LogEdtAppend("\r\n");
		StopBtnClick();

		return;
	}
	SetFileAttributes("C:\\GRLDR", FILE_ATTRIBUTE_HIDDEN
							| FILE_ATTRIBUTE_READONLY
							| FILE_ATTRIBUTE_SYSTEM);


	LogEdtAppend(GetResStr(IDS_DONE));

	InstallPgbSet(95);

	if (!isInstalling) {
		return;
	}

	LogEdtAppend(GetResStr(IDS_MENULST));

	SetFileAttributes("C:\\menu.lst", FILE_ATTRIBUTE_NORMAL);
	f = fopen("C:\\menu.lst", "a+");
	if (!f) {
		LogEdtAppend(GetResStr(IDS_FAILOPEN));
		StopBtnClick();
		return;
	}
	fprintf(f, "\ntitle Puppy Linux (pup2usb)\n");
	fprintf(f, "root %s\n", DriveGrubStr(drv));
	fprintf(f, "kernel /puppy/vmlinuz PMEDIA=idehd PSUBDIR=puppy\n");
	fprintf(f, "initrd /puppy/initrd.gz\n");
	fclose(f);
	SetFileAttributes("C:\\menu.lst", FILE_ATTRIBUTE_HIDDEN
							| FILE_ATTRIBUTE_READONLY
							| FILE_ATTRIBUTE_SYSTEM);

	LogEdtAppend(GetResStr(IDS_DONE));

	if (!isInstalling) {
		return;
	}

	LogEdtAppend(GetResStr(IDS_BOOTINI));

	SetFileAttributes("C:\\boot.ini", FILE_ATTRIBUTE_NORMAL);

    f = fopen("C:\\boot.ini", "r+");
    while (!feof(f)) {
        pos = ftell(f);
        fgets(buf, sizeof(buf), f);
        if (memcmp(buf, "timeout=0", 9) == 0 && strcspn(buf + 9, " \t\r\n") == 0) {
            fseek(f, pos + 8, SEEK_SET);
            fputc('5', f);
            break;
        }
    }
    fclose(f);

	f = fopen("C:\\boot.ini", "a+");
	if (!f) {
		LogEdtAppend(GetResStr(IDS_FAILOPEN));
		StopBtnClick();
		return;
	}
	fprintf(f, "\nC:\\GRLDR=\"Puppy Linux (pup2usb)\"\n");
	fclose(f);
	SetFileAttributes("C:\\boot.ini", FILE_ATTRIBUTE_HIDDEN
							| FILE_ATTRIBUTE_READONLY
							| FILE_ATTRIBUTE_SYSTEM);

	LogEdtAppend(GetResStr(IDS_DONE));

	InstallPgbSet(100);

	return;
}


static DWORD WINAPI InstallThdFunc(LPVOID lpParam)
{
	char	drv;
	char	*iso;

	drv	= DriveCboGet();
	iso = IsoEdtGet();

	if (isInstalling && FormatChkGet()) {
		FormatDrive(drv);
	}
	if (isInstalling) {
		ExtractISO(iso, drv);
	}
	if (isInstalling) {
		if (DriveGetType(drv) == DRIVE_FIXED) {
			MakeBootGrub4Dos(drv);
		} else {
			MakeBootSysLinux(drv);
		}
	}
	if (isInstalling) {
		LogEdtAppend(GetResStr(IDS_FINISHED));
		StopBtnClick();
	} else {
		LogEdtAppend(GetResStr(IDS_STOPPED));
	}

	return 0;
}


static BOOL InstallDlgInit()
{
    DWORD   dwThdId;
	RECT	rc;
	HANDLE  hThd;

	SetWindowText(hwndInstall, GetResStr(IDS_DLG_INSTALL));

	SendDlgItemMessage(
        hwndInstall,
        IDC_EDT_LOG,
        WM_SETFONT,
        (WPARAM)GetStockObject(SYSTEM_FIXED_FONT),
        0);

	GetClientRect(hwndInstall, &rc);
	InstallDlgLayout(rc.right - rc.left, rc.bottom - rc.top);

	isInstalling = TRUE;
	SetDlgItemText(hwndInstall, IDC_BTN_STOP, GetResStr(IDS_BTN_STOP));

	SendDlgItemMessage(
        hwndInstall,
        IDC_PGB_INSTALL,
        PBM_SETRANGE,
        0,
        MAKELPARAM(0, 100));
	SendDlgItemMessage(hwndInstall, IDC_PGB_INSTALL, PBM_SETPOS, 0, 0);

	hThd = CreateThread(
			NULL,
			0,
			InstallThdFunc,
			NULL,
			0,
			&dwThdId);

	if (!hThd)
	{
		ErrorHandler("CreateThread");
		ExitProcess(3);
	}

	return TRUE;
}


static BOOL CALLBACK InstallDlgProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
        case WM_INITDIALOG:
            hwndInstall = hwndDlg;
            InstallDlgInit();
            return TRUE;

        case WM_COMMAND:
            switch (LOWORD(wParam)) {
                case IDC_BTN_STOP:
                    if (isInstalling) {
                        StopBtnClick();
                    } else {
                        EndDialog(hwndDlg, 1);
                    }
                    return TRUE;
            }
            break;

        case WM_SIZE:
            InstallDlgLayout(LOWORD(lParam), HIWORD(lParam));
            return TRUE;

        case WM_CLOSE:
            StopBtnClick();
            EndDialog(hwndDlg, 0);
            return TRUE;
	}
	return FALSE;
}


WNDPROC BarbieImgProcOld;


static LRESULT CALLBACK BarbieImgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static BOOL tracking = FALSE;
    static BOOL hovering = FALSE;

    switch(uMsg)
    {
        case WM_LBUTTONDOWN:
            ShellExecute(
                hwnd,
                "open",
                "http://puppy.cnbits.com/node/120",
                NULL,
                NULL,
                SW_SHOWNORMAL);
            break;

        case WM_MOUSEMOVE:
            if (!tracking) {
                TRACKMOUSEEVENT tme;

                tme.cbSize = sizeof(tme);
                tme.dwFlags = TME_LEAVE;
                tme.hwndTrack = hwnd;
                tme.dwHoverTime = HOVER_DEFAULT;

                TrackMouseEvent(&tme);

                tracking = TRUE;
            }

            if (!hovering) {
                LONG style;
                SetCursor(LoadCursor(NULL, IDC_HAND));

                SendDlgItemMessage(hwndMain, IDC_IMG_BARBIE,
                    STM_SETIMAGE,
                    (WPARAM)IMAGE_BITMAP,
                    (LPARAM)LoadBitmap(hInst, MAKEINTRESOURCE(IDR_BARBIE_HOVER)));
//                SendDlgItemMessage(hwndMain, IDC_IMG_BARBIE,
//                    STM_SETICON,
//                    (WPARAM)LoadIcon(hInst, MAKEINTRESOURCE(IDR_ICON)),
//                    0);


                //SetClassLong(hwnd ,GCL_STYLE,CS_HREDRAW | CS_VREDRAW);
//                if (SendMessage(hwnd,
//                        STM_SETICON,
//                        (WPARAM)IMAGE_ICON,
//                        (LPARAM)LoadIcon(hInst, MAKEINTRESOURCE(IDR_ICON))) == NULL) {
//                    printf("asfasdf\n");
//                };

//                SendMessage(hwnd, BM_SETSTYLE, (WPARAM)LOWORD(BS_FLAT), MAKELPARAM(TRUE, 0));
//                SetWindowLong(hwnd,
//                    GWL_STYLE, 0x5080030E);
//                    GetWindowLong(hwnd, GWL_STYLE) | 0x800000);
                hovering = TRUE;
            }
//            printf("move\n");
            break;

        case WM_MOUSELEAVE:
            SetCursor(LoadCursor(NULL, IDC_ARROW));

            SendDlgItemMessage(hwndMain, IDC_IMG_BARBIE,
                STM_SETIMAGE,
                (WPARAM)IMAGE_BITMAP,
                (LPARAM)LoadBitmap(hInst, MAKEINTRESOURCE(IDR_BARBIE)));

            tracking = FALSE;
            hovering = FALSE;
//            printf("leave\n");
            break;

        case WM_SETCURSOR:
            return TRUE;

//        default:
//            printf("%d\n", uMsg);
//            break;
    }

    return BarbieImgProcOld(hwnd, uMsg, wParam, lParam);
}


static int MainDlgInit()
{
	DriveDebug();

    BarbieImgProcOld = SetWindowLong(GetDlgItem(hwndMain, IDC_IMG_BARBIE), GWL_WNDPROC, BarbieImgProc);
    if (BarbieImgProcOld == 0) {
        ErrorHandler("SetWindowLong");
        ExitProcess(3);
    }

	SetClassLong(
		hwndMain,
		GCL_HICON,
		(LONG)LoadIcon(hInst, MAKEINTRESOURCE(IDR_ICON16)));

    SendDlgItemMessage(hwndMain, IDC_BTN_HELP,
        BM_SETIMAGE,
        IMAGE_ICON,
        (WPARAM)LoadImage(hInst, MAKEINTRESOURCE(IDR_HELP), IMAGE_ICON, 20, 20, 0));

    SendDlgItemMessage(hwndMain, IDC_IMG_BARBIE,
        STM_SETIMAGE,
        (WPARAM)IMAGE_BITMAP,
        (LPARAM)LoadBitmap(hInst, MAKEINTRESOURCE(IDR_BARBIE)));

//    SendDlgItemMessage(hwndMain, IDC_IMG_BARBIE,
//        STM_SETICON,
//        (WPARAM)LoadIcon(hInst, MAKEINTRESOURCE(IDR_ICON16)),
//        0);


//	SendDlgItemMessage(hwndMain,
//        IDC_BTN_BARBIE,
//        BM_SETIMAGE,
//        IMAGE_ICON,
//        (LPARAM)LoadIcon(hInst, MAKEINTRESOURCE(IDR_ICON)));

	SetWindowText(hwndMain, GetResStr(IDS_DLG_MAIN));
	SetDlgItemText(hwndMain, IDC_GRP_ISO, GetResStr(IDS_GRP_ISO));
	SetDlgItemText(hwndMain, IDC_STC_ISO, GetResStr(IDS_STC_ISO));
	SetDlgItemText(hwndMain, IDC_GRP_DRIVE, GetResStr(IDS_GRP_DRIVE));
	SetDlgItemText(hwndMain, IDC_STC_DRIVE, GetResStr(IDS_STC_DRIVE));
	SetDlgItemText(hwndMain, IDC_BTN_BROWSE, GetResStr(IDS_BTN_BROWSE));
	SetDlgItemText(hwndMain, IDC_CHK_FORMAT, GetResStr(IDS_CHK_FORMAT));
	SetDlgItemText(hwndMain, IDC_BTN_START, GetResStr(IDS_BTN_START));
	SetDlgItemText(hwndMain, IDC_BTN_EXIT, GetResStr(IDS_BTN_EXIT));
	SetFocus(GetDlgItem(hwndMain, IDC_EDT_ISO));
	DriveCboRefresh(hwndMain);

	return 1;
}


BOOL CALLBACK MainDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            hwndMain = hwndDlg;
            MainDlgInit();
            return TRUE;

        case WM_CLOSE:
            EndDialog(hwndDlg, 0);
            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case IDC_BTN_START:
                    if (MainDlgCheck()) {
                        return DialogBox(NULL,
                                MAKEINTRESOURCE(DLG_INSTALL),
                                hwndDlg,
                                (DLGPROC)InstallDlgProc);
                    }
                    break;
                case IDC_BTN_EXIT:
                    EndDialog(hwndDlg,0);
                    return TRUE;
                case IDC_BTN_BROWSE:
                    BrowseBtnClick();
                    break;
                case IDC_BTN_HELP:
                    MessageBox(hwndDlg, GetResStr(IDS_HELPTEXT), GetResStr(IDS_HELP), MB_OK);
                    break;
                case IDC_CBO_DRIVE:
                    DriveCboProc(HIWORD(wParam));
                    break;
            }
            return TRUE;
    }

    return FALSE;
}


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
    hInst = hInstance;

    // The user interface is a modal dialog box
    return DialogBox(hInstance, MAKEINTRESOURCE(DLG_MAIN), NULL, MainDlgProc);
}
