#include <windows.h>
#include <stdio.h>
#include "util.h"


char *GetLastErrorStr(void)
{
	static char	buf[BUFSIZE];
	LPVOID		msg;

	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&msg,
		0, NULL );

	strncpy(buf, msg, BUFSIZE);
	LocalFree(msg);

	return buf;
}


void ErrorHandler(LPTSTR lpszFunction)
{
	char buf[BUFSIZE];

	snprintf(buf, sizeof(buf),
		"%s (%u): %s",
		lpszFunction, (UINT)GetLastError(), GetLastErrorStr());

	MessageBox(NULL, buf, "Error", MB_ICONERROR | MB_OK);
}


char *GetResStr(UINT uID)
{
    static int  n = 0;
    static char buf[16][BUFSIZE];

    n = (n + 1) % (sizeof(buf) / sizeof(buf[0]));
    if (!LoadString(GetModuleHandle(NULL), uID, buf[n], sizeof(buf[n]))) {
        ErrorHandler("LoadString");
        ExitProcess(3);
    }

    return buf[n];
}


HANDLE POpen(const char *cmd, PPROCESS_INFORMATION pi, BOOL isErr)
{
	STARTUPINFO			si;
	SECURITY_ATTRIBUTES	sa;
	HANDLE				hOut;
	HANDLE				hErr;

	GetStartupInfo(&si);
	si.dwFlags				= STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
	si.wShowWindow			= SW_HIDE;
	sa.bInheritHandle		= TRUE;
	sa.lpSecurityDescriptor	= NULL;
	sa.nLength				= sizeof(SECURITY_ATTRIBUTES);

	if (!CreatePipe(&hOut, &hErr, &sa, 0)) {
		ErrorHandler("CreatePipe");
		ExitProcess(3);
	}

	if (isErr) {
		si.hStdError		= hErr;
	} else {
		si.hStdOutput		= hErr;
	}

	if (!CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &si, pi)) {
		ErrorHandler("CreateProcess");
		ExitProcess(3);
	}

	CloseHandle(hErr);

	return hOut;
}


char* ReadPipe(HANDLE hOut)
{
	char	buf[1024];
	char	*p;
	int		psize;
	int		plen;
	int		len;

	plen	= 0;
	psize	= sizeof(buf);
	p		= (char *)HeapAlloc(GetProcessHeap(), 0, psize);
	if (!p) {
		ExitProcess(2);
	}
	*p		= 0;

	while (TRUE) {
		if (ReadFile(hOut, buf, sizeof(buf) - 1 , &len, NULL)) {
			if (len == 0) {
				MessageBox(NULL, "0", "0", 0);
				break;
			}
			buf[len] = 0;
			if (plen + len + 1 > psize) {
				psize	*= 2;
				p		= (char *)HeapReAlloc(GetProcessHeap(), 0, p, psize);
			}
			memcpy(p + plen, buf, len + 1);
			plen += len;
		} else {
			if (GetLastError() == ERROR_BROKEN_PIPE) {
				break;
			}
			ErrorHandler("ReadFile");
			ExitProcess(3);
		}
	}

	CloseHandle(hOut);

	return p;
}


int System(const char *cmd)
{
    PROCESS_INFORMATION pi;
    STARTUPINFO			si;
    int                 ret;

	ZeroMemory(&si, sizeof(si));
	si.dwFlags				= STARTF_USESHOWWINDOW;
	si.wShowWindow			= SW_HIDE;

	if (!CreateProcess(NULL, cmd, NULL, NULL, FALSE,
            CREATE_NEW_CONSOLE, NULL, NULL, &si, &pi)) {
		ErrorHandler("CreateProcess");
		ExitProcess(3);
	}
    WaitForSingleObject(pi.hProcess, INFINITE);
    if(!GetExitCodeProcess(pi.hProcess, &ret)) {
        ErrorHandler("GetExitCodeProcess");
        ExitProcess(3);
    }
   	CloseHandle(pi.hThread);
	CloseHandle(pi.hProcess);

	return ret;
}


UINT FileSize(const char *name)
{
	HANDLE	f;
	DWORD	high;
	DWORD	low;

   	f = CreateFile(
			name,
			0,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_READONLY,
			NULL);

	if (f == INVALID_HANDLE_VALUE) {
		return 0;
	}

	low = GetFileSize(f, &high);

	CloseHandle(f);

	if (low == 0xFFFFFFFF) {
		return 0;
	}

	return low;
}
